/*
 * SupportWizard 2
 * 
 * Copyright (C) 2001 ISC corp. All Rights Reserved.
 * 
 * $Id: SWFunctionality.java,v 1.16 2003/07/22 23:05:30 alexi Exp $
 * 
 * Created by Konstantin Sobolev.
 * Date: Jul 9, 2001
 * Time: 9:22:11 PM
 */
package com.supportwizard.functionalities;

import com.supportwizard.seance.Seanceable;
import javax.ejb.EJBObject;
import java.rmi.RemoteException;

/**
 * Basic functionality interface. All real functionalities
 * remote interfaces must implement it.
 * 
 * @see com.supportwizard.functionalities.SWFunctionalityBase
 * 
 * @author <a href="mailto:kos@supportwizard.com">Konstantin Sobolev</a>
 * @version $Revision: 1.16 $
 */
public interface SWFunctionality extends EJBObject, Seanceable {

  /**
   * Functionalities may be temporarily disabled. This is useful when you need
   * to modify some tables from functionality callback. Disabling functionality
   * in such case will avoid internal recursion.
   * 
   * @return <code>true</code> if functionality is enabled.
   */
  boolean isEnabled() throws RemoteException;

  /**
   * Functionalities may be temporarily disabled. This is useful when you need
   * to modify some tables from functionality callback. Disabling functionality
   * in such case will avoid internal recursion.
   * 
   * @param enabled New functionality state.
   */
  void setEnabled(boolean enabled) throws RemoteException;

  /**
   * Gets functionality name.
   * 
   * @return Name of this functionality.
   */
  String getFunctionalityName() throws RemoteException;

  /**
   * Gets functionality description.
   * 
   * @return Description of this functionality.
   */
  String getFunctionalityDescription() throws RemoteException;

  /**
   * Gets functionality ID.
   * 
   * @return Functionality ID.
   */
  Long getFunctionalityID() throws RemoteException;

  /**
   * Gets ID of SWTable this functionality is associated with.
   * 
   * @return SWTable ID
   */
  Long getSWTableID() throws RemoteException;

  /**
   * Gets functionality metadata ID.
   * 
   * @return Functionality metadata ID.
   */
  Long getFunctionalityMetadataID() throws RemoteException;

  /**
   * Gets functionality logic.
   * 
   * @return Functionality logic instance.
   */
  SWFunctionalityLogic getFunctionalityLogic() throws RemoteException;

  /**
   * @link dependency
   */

  /* #SWFunctionalityLogic lnkSWFunctionalityLogic; */
}

