/*
 * SupportWizard 2
 * 
 * Copyright (C) 2001 ISC corp. All Rights Reserved.
 * 
 * $Id: SWFunctionalityCreationInfo.java,v 1.7 2003/07/22 23:05:30 alexi Exp $
 * Created by Konstantin Sobolev (kos@supportwizard.com)
 * Date: Sep 15, 2001
 * Time: 12:26:40 PM
 * 
 * Last modification $Date: 2003/07/22 23:05:30 $
 */
package com.supportwizard.functionalities;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * Auxiliary class that is needed by <code>SWFuctionalityBase</code> during functionality
 * creation. Contains almost all data (except for target <code>SWTable</code> ID) that is necessary to create the
 * functionality, it's parameters and column mappings.
 * 
 * @author Konstantin Sobolev (kos@supportwizard.com)
 * @revision $Revision: 1.7 $
 */
public final class SWFunctionalityCreationInfo {
  private ArrayList paramsCreationInfo;
  private ArrayList mappingsCreationInfo;

  /**
   * Functionality name.
   */
  public String     funcName;

  /**
   * Functionality description.
   */
  public String     funcDescription;

  /**
   * Functionality home class name.
   */
  public String     funcHomeClassName;

  /**
   * Functionality home JNDI name.
   */
  public String     funcHomeJndiName;

  /**
   * Creates new <code>SWFunctionalityCreationInfo</code> instance with all fields set to <code>null</code> and
   * empty list of parameters creation info.
   */
  public SWFunctionalityCreationInfo() {
    this.funcName = null;
    this.funcDescription = null;
    this.funcHomeClassName = null;
    this.funcHomeJndiName = null;
    paramsCreationInfo = new ArrayList();
    mappingsCreationInfo = new ArrayList();
  }

  /**
   * Creates new <code>SWFunctionalityCreationInfo</code> instance with empty list of parameters creation info.
   * 
   * @param funcName Functionality name.
   * @param funcDescription Functionality description.
   * @param funcHomeClassName Functionality home class name.
   * @param funcHomeJndiName Functionality home JNDI name.
   */
  public SWFunctionalityCreationInfo(String funcName, String funcDescription, String funcHomeClassName, 
                                     String funcHomeJndiName) {
    this.funcName = funcName;
    this.funcDescription = funcDescription;
    this.funcHomeClassName = funcHomeClassName;
    this.funcHomeJndiName = funcHomeJndiName;
    paramsCreationInfo = new ArrayList();
    mappingsCreationInfo = new ArrayList();
  }

  /**
   * Adds new <code>SWFuncParameterCreationInfo</code>.
   * 
   * @param parameterCreationInfo Instance to be added.
   */
  public void addParameterCreationInfo(SWFuncParameterCreationInfo parameterCreationInfo) {
    paramsCreationInfo.add(parameterCreationInfo);
  } 

  /**
   * Gets <code>SWFuncParameterCreationInfo</code> with specified index. Valid indices are
   * <code>0...getNumberOfParameters()-1</code>.
   * 
   * @param i Index of <code>SWFuncParameterCreationInfo</code> to be returned.
   * @return <code>SWFuncParameterCreationInfo</code> at given index.
   */
  SWFuncParameterCreationInfo getParameterCreationInfo(int i) {
    return ((SWFuncParameterCreationInfo) paramsCreationInfo.get(i));
  } 

  /**
   * Gets number of available parameters info.
   */
  int getNumberOfParameters() {
    return paramsCreationInfo.size();
  } 

  /**
   * Adds new <code>SWFuncColsMappingCreationInfo</code>.
   * 
   * @param mappingCreationInfo Instance to be added.
   */
  public void addMappingCreationInfo(SWFuncColsMappingCreationInfo mappingCreationInfo) {
    mappingsCreationInfo.add(mappingCreationInfo);
  } 

  /**
   * Gets <code>SWFuncColsMappingCreationInfo</code> with specified index. Valid indices are
   * <code>0...getNumberOfMappings()-1</code>.
   * 
   * @param i Index of <code>SWFuncColsMappingCreationInfo</code> to be returned.
   * @return <code>SWFuncColsMappingCreationInfo</code> at given index.
   */
  SWFuncColsMappingCreationInfo getMappingCreationInfo(int i) {
    return ((SWFuncColsMappingCreationInfo) mappingsCreationInfo.get(i));
  } 

  /**
   * Gets number of available mappings info.
   */
  int getNumberOfMappings() {
    return mappingsCreationInfo.size();
  } 

}

