/*
 * $Id: SWAccessFunctionalityBean.java,v 1.4 2003/07/22 23:05:31 alexi Exp $
 * 
 * Created by Michael Degtyarev (dmv@supportwizard.com)
 * Created: Apr. 13, 2003
 */
package com.supportwizard.functionalities.access.ejb;

import com.supportwizard.dictionary.SWColumn;
import com.supportwizard.dictionary.SWTable;
import com.supportwizard.functionalities.SWFunctionalityBase;
import com.supportwizard.functionalities.SWFunctionalityCreationInfo;
import com.supportwizard.functionalities.SWFunctionalityLogic;
import com.supportwizard.functionalities.access.SWAccessFunctionalityConst;
import com.supportwizard.functionalities.access.SWAccessFunctionalityLogic;
import org.apache.log4j.Logger;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import java.rmi.RemoteException;
import java.util.Collection;

/**
 * Access functionality.
 * 
 * @author Michael Degtyarev
 * @version $Revision: 1.4 $
 * 
 * @ejb:bean name="SWAccessFunctionality" type="BMP" jndi-name="functionalities/SWAccessFunctionality" transaction-type="Container"
 * @ejb:data-object container="false"
 * @ejb:pk class="java.lang.Long"
 * @ejb:interface extends="javax.ejb.EJBObject, com.supportwizard.functionalities.SWFunctionality"
 * @ejb:resource-ref res-name="jdbc/sw" res-type="javax.sql.DataSource" res-auth="Container"
 * 
 * @ejb:ejb-external-ref ref-name="dictionary/SWTable" type="Entity" home="com.supportwizard.dictionary.SWTableHome" remote="com.supportwizard.dictionary.SWTable"
 * @jboss:ejb-ref-jndi ref-name="dictionary/SWTable" jndi-name="dictionary/SWTable"
 * @ejb:ejb-external-ref ref-name="dictionary/SWColumn" type="Entity" home="com.supportwizard.dictionary.SWColumnHome" remote="com.supportwizard.dictionary.SWColumn"
 * @jboss:ejb-ref-jndi ref-name="dictionary/SWColumn" jndi-name="dictionary/SWColumn"
 * @ejb:ejb-external-ref ref-name="dml/SWRecord" type="Entity" home="com.supportwizard.dml.SWRecordHome" remote="com.supportwizard.dml.SWRecord"
 * @jboss:ejb-ref-jndi ref-name="dml/SWRecord" jndi-name="dml/SWRecord"
 * @ejb:ejb-external-ref ref-name="dml/SWData" type="Session" home="com.supportwizard.dml.SWDataHome" remote="com.supportwizard.dml.SWData"
 * @jboss:ejb-ref-jndi ref-name="dml/SWData" jndi-name="dml/SWData"
 */
public abstract class SWAccessFunctionalityBean extends SWFunctionalityBase implements javax.ejb.EntityBean {
  private static final Logger log = Logger.getLogger(SWAccessFunctionalityBean.class.getName());

  /**
   * Create new access functionality.
   * 
   * @param table The SWTable this functionality will be associated with.
   * 
   * @ejb:create-method
   */
  public Long ejbCreate(SWTable table, SWColumn column) throws CreateException {
    log.debug("ENTERING SWAccessFunctionality.ejbCreate(" + table + "," + column + ")");

    try {

      // Check whether we do not already have OwnerFunctionality for this table.
      try {
        Collection ownerFunc = ejbFindByTable(table);

        if (ownerFunc.isEmpty() == false) {
          throw new CreateException("There can be only one SWAccessFunctionality per table installed.");
        } 
      } catch (FinderException e) {}

      // Check whether  the column belongs to this table.
      if (!table.getPrimaryKey().equals(column.getSWTable().getPrimaryKey())) {
        throw new CreateException("Column doesn't belong to the table");
      } 

      funcID = super.ejbCreate(table);

      log.debug("Parent method successfully executed:" + funcID);

      Long swColumnID = column.getSWColumnPK().getSWColumnID();

      getFuncColsMappingHome().create(funcID, table.getSWTableID(), swColumnID, 
                                      SWAccessFunctionalityConst.ACCESSFLAG_COLUMN_ALIAS);

      return funcID;
    } catch (RemoteException ex) {
      log.error("RemoteException occured during ejbCreate()", ex);

      throw new CreateException(ex.getMessage());
    } 
  } 

  public void ejbPostCreate(SWTable table, SWColumn column) throws CreateException {}

  /**
   * @return Functionality logic instance.
   * 
   * ejb:interface-method view-type="remote"
   */
  public SWFunctionalityLogic getFunctionalityLogic() {
    try {
      Long functionalityID = super.getFunctionalityID();
      Long swTableID = super.getSWTableID();

      return new SWAccessFunctionalityLogic(functionalityID, swTableID);
    } catch (Exception ex) {
      throw new EJBException("Failed to retrieve functionality logic", ex);
    } 
  } 

  /**
   * Gets functionality creation info.
   * 
   * @return Functionality creation info/
   */
  protected SWFunctionalityCreationInfo getFunctionalityCreationInfo() {
    SWFunctionalityCreationInfo res = new SWFunctionalityCreationInfo();

    res.funcName = SWAccessFunctionalityConst.FUNC_NAME;
    res.funcDescription = SWAccessFunctionalityConst.FUNC_DESCRIPTION;
    res.funcHomeClassName = SWAccessFunctionalityConst.FUNC_HOME_CLASS_NAME;
    res.funcHomeJndiName = SWAccessFunctionalityConst.FUNC_HOME_JNDI_NAME;

    return res;
  } 

}

