/*
 * SupportWizard 2
 * 
 * Copyright (C) 2002 ISC corp. All Rights Reserved.
 * 
 * $Id: AHstTableManager.java,v 1.8 2003/07/22 23:05:31 alexi Exp $
 * Created by Konstantin Sobolev (kos@supportwizard.com) on Jun 23, 2002
 * Last modification $Date: 2003/07/22 23:05:31 $
 */
package com.supportwizard.functionalities.attributehistory;

import com.supportwizard.dictionary.SWTable;

// bedal: needed only in commentaries
// import com.supportwizard.functionalities.attributehistory.ejb.SWAttributeHistoryLogRecordBean;
import javax.ejb.EJBException;

/**
 * Utility class for managing _ahst_ tables. Mainly used by <code>SWAttributeHistoryFunctionalityBean</code>.
 * <p>
 * _ahst_ table is used for storing <code>SWAttributeHistoryLogRecordBean</code> data and has the following layout:
 * <ul>
 * <li>BIGINT RecordID  -- Autoincrementing primary key.</li>
 * <li>BIGINT ActionDate -- Action date in milliseconds.</li>
 * <li>BIGINT UserID -- Actor's ID.</li>
 * <li>{set of fields that make up user table's primary key with '_pk_' suffixes in their names} -- used for
 * storing <code>SWRecordPK</code></li>
 * </ul>
 * 
 * @author <a href="mailto:kos@supportwizard.com" title="">Konstantin Sobolev</a>
 * @version $Revision: 1.8 $
 * @see com.supportwizard.functionalities.attributehistory.ejb.SWAttributeHistoryLogRecordBean
 */
public interface AHstTableManager {

  /**
   * Creates new _ahst_ table.
   * 
   * @param table Target user table.
   * 
   * @throws EJBException in case of run-time error.
   */
  void createHstTable(SWTable table);

  /**
   * Gets the number of records in an _ahst_ table.
   * 
   * @return Number of records in _ahst_ table.
   * 
   * @throws EJBException in case of run-time error.
   */
  int getAHstTableSize(Long tableID);

  /**
   * Drops _ahst_ table.
   * 
   * @param tableID ID of user table _ahst_ table is created for.
   * 
   * @throws EJBException in case of run-time error.
   */
  void dropAHstTable(Long tableID);

  /**
   * Gets _ahst_ table name.
   * 
   * @param table Target user table.
   * @return _ahst_ table name.
   */
  String getAHstTableName(SWTable table);

  /**
   * Gets _ahst_ table name.
   * 
   * @param tableID ID of target user table.
   * @return _ahst_ table name.
   */
  String getAHstTableName(Long tableID);

  /**
   * Gets tables to drop on remove. This will be used as a result of
   * {@link SWAttributeHistoryFunctionalityLogic#getTablesToDropOnRemove()} method.
   * 
   * @param tableID Target table ID.
   * 
   * @return tables to drop on remove.
   */
  String[] getTablesToDropOnRemove(Long tableID);
}

