/*
 * SupportWizard 2
 * 
 * Copyright (C) 2002 ISC corp. All Rights Reserved.
 * 
 * $Id: DictionaryAHstTableManagerImpl.java,v 1.7 2003/07/22 23:05:31 alexi Exp $
 * Created by Konstantin Sobolev (kos@supportwizard.com) on Oct 31, 2002
 * Last modification $Date: 2003/07/22 23:05:31 $
 */
package com.supportwizard.functionalities.attributehistory;

import com.supportwizard.db.*;
import com.supportwizard.dictionary.*;
import com.supportwizard.seance.Seance;
import com.supportwizard.utils.EJBException2;
import com.supportwizard.utils.HomesGetter;
import org.apache.log4j.Logger;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.naming.NamingException;
import java.rmi.RemoteException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;

/**
 * Dictionary-based {@link AHstTableManager} implementation.
 * 
 * @author <a href="mailto:kos@supportwizard.com" title="">Konstantin Sobolev</a>
 * @version $Revision: 1.7 $
 */
public class DictionaryAHstTableManagerImpl implements AHstTableManager {
  private static final Logger log = Logger.getLogger(DictionaryAHstTableManagerImpl.class);
  private SWDictionaryDDL     dictionaryDDL;
  private SWTableHome         tableHome;

  // private Seance seance;
  public DictionaryAHstTableManagerImpl(Seance seance) throws NamingException, RemoteException, CreateException {

    // this.seance = seance;
    HomesGetter         homesGetter = (seance == null ? new HomesGetter() : seance.getHomesGetter());
    SWDictionaryDDLHome dictionaryDDLHome = (SWDictionaryDDLHome) homesGetter.getHome(SWDictionaryDDLHome.class, 
            "dictionary/SWDictionaryDDL");

    tableHome = (SWTableHome) homesGetter.getHome(SWTableHome.class, "dictionary/SWTable");
    dictionaryDDL = dictionaryDDLHome.create();
  }

  protected void finalize() throws Throwable {
    super.finalize();
    dictionaryDDL.remove();
  } 

  /**
   * Creates new _ahst_ table.
   * 
   * @param table Target user table.
   * 
   * @throws EJBException in case of run-time error.
   */
  public void createHstTable(SWTable table) {
    try {
      dictionaryDDL.createAttributeHistoryTable(table);
    } catch (Exception e) {
      log.error("Can't create attribute history table for table = " + table, e);
    } 
  } 

  /**
   * Gets the number of records in an _ahst_ table.
   * 
   * @return Number of records in _ahst_ table.
   * 
   * @throws EJBException in case of run-time error.
   */
  public int getAHstTableSize(Long tableID) {
    int            res = -1;
    SWInternal_SQL sw_sql = null;

    try {
      sw_sql = getSQLManager();
    } catch (SW_SQLException e) {
      log.error("Couldn't get DB connection", e);

      throw new EJBException(e);
    } 

    try {
      TableList tableList = new TableList();

      tableList.add(getAHstTableName(tableID), "");

      ColumnList columnList = new ColumnList();

      columnList.add("*", "", "", "count");

      HashMap params = new HashMap();

      params.put("from", tableList);  // select count(*) from blablabla_ahst_
      params.put("columns", columnList);
      params.put("where", new ConditionList());

      ResultSet rs = sw_sql.sqlQuery(params);

      rs.next();

      res = rs.getInt(1);

      rs.close();
    } catch (Exception e) {
      log.debug("Exception caught", e);

      throw new EJBException2(e);
    } 
    finally {
      try {
        sw_sql.dbDisconnect();
      } catch (SQLException e) {
        log.error("Cant't close database connection", e);
      } 
    } 

    return res;
  } 

  /**
   * Drops _ahst_ table.
   * 
   * @param tableID ID of user table _ahst_ table is created for.
   * 
   * @throws EJBException in case of run-time error.
   */
  public void dropAHstTable(Long tableID) {
    try {

      // this works, but breaks transaction rollback
      // SWTable table = tableHome.findByPrimaryKey(tableID);
      // dictionaryDDL.dropTable(table.getAttributeHistoryTableID().longValue());
      // table.setAttributeHistoryTableID(null);
      SWTable table = tableHome.findByPrimaryKey(tableID);
      SWTable ahstTable = tableHome.findByPrimaryKey(table.getAttributeHistoryTableID());

      ahstTable.remove();
      table.setAttributeHistoryTableID(null);
    } catch (Exception e) {
      log.error("Can't drop attribue history column from table " + tableID, e);
    } 
  } 

  /**
   * Gets _ahst_ table name.
   * 
   * @param table Target user table.
   * @return _ahst_ table name.
   */
  public String getAHstTableName(SWTable table) {
    try {
      return (tableHome.findByPrimaryKey(table.getAttributeHistoryTableID())).getDBName();
    } catch (Exception e) {
      log.error("Can't get attribue history table name: " + e, e);
    } 

    return null;
  } 

  /**
   * Gets _ahst_ table name.
   * 
   * @param tableID ID of target user table.
   * @return _ahst_ table name.
   */
  public String getAHstTableName(Long tableID) {
    try {
      return getAHstTableName(tableHome.findByPrimaryKey(tableID));
    } catch (Exception e) {
      log.error("Can't get attribue history table name: " + e, e);
    } 

    return null;
  } 

  /**
   * Gets tables to drop on remove. This will be used as a result of
   * {@link SWAttributeHistoryFunctionalityLogic#getTablesToDropOnRemove()} method.
   * 
   * @param tableID Target table ID.
   * 
   * @return tables to drop on remove.
   */
  public String[] getTablesToDropOnRemove(Long tableID) {

    // return new String[0];
    return new String[] {
      getAHstTableName(tableID)
    };
  } 

  private SWInternal_SQL getSQLManager() throws SW_SQLException {
    final SWInternal_SQL sw_sql;

    sw_sql = new SWInternal_SQL();

    sw_sql.dbConnect();

    return sw_sql;
  } 

}

