/*
 * SupportWizard 2
 * 
 * Copyright (C) 2002 ISC corp. All Rights Reserved.
 * 
 * $Id: SWAttributeChangeBean.java,v 1.8 2003/07/22 23:05:31 alexi Exp $
 * Created by Konstantin Sobolev (kos@supportwizard.com) on 18.06.2002
 * Last modification $Date: 2003/07/22 23:05:31 $
 */
package com.supportwizard.functionalities.attributehistory.ejb;

import com.supportwizard.functionalities.attributehistory.interfaces.SWAttributeChangePK;
import javax.ejb.EntityBean;
import javax.ejb.CreateException;

/**
 * Represents single attribute change: attribute name, it's old and new values.
 * Maps to the following <code>swattributechange</code> table:
 * 
 * <pre>
 * +----------+-------------+------+-----+---------+-------+---------------------------------+
 * | Field    | Type        | Null | Key | Default | Extra | Privileges                      |
 * +----------+-------------+------+-----+---------+-------+---------------------------------+
 * | RecordID | bigint(20)  |      |     | 0       |       | select,insert,update,references |
 * | AttrName | varchar(99) |      |     |         |       | select,insert,update,references |
 * | OldValue | varchar(99) | YES  |     |         |       | select,insert,update,references |
 * | NewValue | varchar(99) | YES  |     |         |       | select,insert,update,references |
 * +----------+-------------+------+-----+---------+-------+---------------------------------+
 * </pre>
 * 
 * <code>RecordID</code> refers to record id from _ahst_ table. See
 * {@link SWAttributeHistoryLogRecordBean SWAttributeHistoryLogRecord} for more detail on _ahst_ table.
 * 
 * @author <a href="mailto:kos@supportwizard.com" title="">Konstantin Sobolev</a>
 * @version $Revision: 1.8 $
 * 
 * @ejb:bean name="SWAttributeChange" type="CMP" jndi-name="functionalities/attributehistory/SWAttributeChange" transaction-type="Container"
 * @ejb:transaction type="Supports"
 * @ejb:data-object container="false"
 * @ejb:finder signature="java.util.Collection findByRecordID(java.lang.Long recordID)"
 * query="SELECT Object(o) FROM SWAttributeChange o WHERE o.recordID = ?1"
 * result-type-mapping="Remote"
 * @jboss:finder-query name="findByRecordID" query="RecordID = {0}"
 * 
 * @jboss:table-name "swattributechange"
 */
public abstract class SWAttributeChangeBean implements EntityBean {

  /**
   * Gets record ID.
   * 
   * @return Record ID.
   * 
   * @ejb:persistent-field
   * @ejb:interface-method
   * @ejb:pk-field
   * 
   * @jboss:column-name "RecordID"
   */
  public abstract Long getRecordID();

  /**
   * Sets <code>SWAttributeHistoryLogRecord</code> record ID.
   * 
   * @param recordID record ID.
   */
  public abstract void setRecordID(Long recordID);

  /**
   * Gets attribute name.
   * 
   * @return Attribute name.
   * 
   * @ejb:persistent-field
   * @ejb:interface-method
   * @ejb:pk-field
   * 
   * @jboss:column-name "AttrName"
   */
  public abstract String getAttributeName();

  /**
   * Sets attribute name.
   * 
   * @param attributeName attribute name.
   */
  public abstract void setAttributeName(String attributeName);

  /**
   * Gets old attribute value.
   * 
   * @return Old value.
   * 
   * @ejb:persistent-field
   * @ejb:interface-method
   * 
   * @jboss:column-name "OldValue"
   */
  public abstract String getOldValue();

  /**
   * Sets old attribute value.
   * 
   * @param oldValue old value.
   */
  public abstract void setOldValue(String oldValue);

  /**
   * Gets new attribute value.
   * 
   * @return New attribute value.
   * 
   * @ejb:persistent-field
   * @ejb:interface-method
   * 
   * @jboss:column-name "NewValue"
   */
  public abstract String getNewValue();

  /**
   * Sets new attribute value.
   * 
   * @param newValue new attribute value.
   */
  public abstract void setNewValue(String newValue);

  /**
   * Creates new <code>SWAttributeChange</code> instance.
   * 
   * @param recordID <code>SWAttributeHistoryLogRecord</code> record ID.
   * @param attributeName attribute name.
   * @param oldValue old attribute value.
   * @param newValue new attribute value.
   * 
   * @return Primary key of the newly created instance.
   * @throws CreateException in case of application-level error.
   * 
   * @ejb:create-method
   * @ejb:transaction type="Required"
   */
  public SWAttributeChangePK ejbCreate(Long recordID, String attributeName, String oldValue, 
                                       String newValue) throws CreateException {
    setRecordID(recordID);
    setAttributeName(attributeName);
    setOldValue(oldValue);
    setNewValue(newValue);

    return null;
  } 

  public void ejbPostCreate(Long recordID, String attributeName, String oldValue, String newValue) {}

}

