/*
 * SupportWizard 2
 *
 * Copyright (C) 2002 ISC corp. All Rights Reserved.
 *
 * $Id: SWAttributeHistoryFunctionalityBean.java,v 1.15 2003/09/03 18:47:57 kos Exp $
 * Created by Konstantin Sobolev (kos@supportwizard.com) on 17.06.2002
 * Last modification $Date: 2003/09/03 18:47:57 $
 */

package com.supportwizard.functionalities.attributehistory.ejb;

import com.supportwizard.db.Types;
import com.supportwizard.db.SW_SQLException;
import com.supportwizard.dictionary.*;
import com.supportwizard.functionalities.SWFuncParameterCreationInfo;
import com.supportwizard.functionalities.SWFunctionalityBase;
import com.supportwizard.functionalities.SWFunctionalityCreationInfo;
import com.supportwizard.functionalities.SWFunctionalityLogic;
import com.supportwizard.functionalities.attributehistory.AHstTableManager;
import com.supportwizard.functionalities.attributehistory.SWAttributeHistoryFunctionalityLogic;
import com.supportwizard.functionalities.attributehistory.AHstTableManagerFactory;
import com.supportwizard.functionalities.attributehistory.interfaces.*;
import com.supportwizard.functionalities.interfaces.SWFuncParameterPK;
import com.supportwizard.utils.EJBException2;
import com.supportwizard.utils.HomesGetter;
import com.supportwizard.seance.Seance;
import org.apache.log4j.Logger;

import javax.ejb.*;
import javax.naming.NamingException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.rmi.RemoteException;

/**
 * Attribe history functionality. Keeps history of attribute modifications.
 * Returns {@link SWAttributeHistoryFunctionalityLogic SWAttributeHistoryFunctionalityLogic}
 * as logic implementaion. Manages (creates and deletes) _ahst_ tables (see
 * {@link SWAttributeHistoryLogRecordBean SWAttributeHistoryLogRecordBean}
 * for more details). For complete description see <a href="../package-summary.html">package documentation</a>.
 *
 * @author <a href="mailto:kos@supportwizard.com" title="">Konstantin Sobolev</a>
 * @version $Revision: 1.15 $
 *
 * @ejb:bean name="SWAttributeHistoryFunctionality" type="BMP" jndi-name="functionalities/attributehistory/SWAttributeHistoryFunctionality" transaction-type="Container"
 * @ejb:transaction type="Supports"
 * @ejb:data-object container="false"
 * @ejb:pk class="java.lang.Long"
 * @ejb:interface extends="javax.ejb.EJBObject, com.supportwizard.functionalities.SWFunctionality"
 * @ejb:resource-ref res-name="jdbc/sw" res-type="javax.sql.DataSource" res-auth="Container"
 *
 * @ejb:ejb-ref ejb-name="SWAttributeChange" ref-name="functionalities/attributehistory/SWAttributeChange"
 * @ejb:ejb-ref ejb-name="SWAttributeHistoryLogRecord"
 *
 * @ejb:ejb-external-ref ejb-name="Seance" type="Session" home="com.supportwizard.seance.SeanceHome" remote="com.supportwizard.seance.Seance"
 * @jboss:ejb-ref-jndi ref-name="Seance" jndi-name="ejb/Seance"
 * @ejb:ejb-external-ref ejb-name="dictionary/SWTable" type="Entity" home="com.supportwizard.dictionary.SWTableHome" remote="com.supportwizard.dictionary.SWTable"
 * @jboss:ejb-ref-jndi ref-name="dictionary/SWTable" jndi-name="dictionary/SWTable"
 * @ejb:ejb-external-ref ejb-name="dictionary/SWColumn" type="Entity" home="com.supportwizard.dictionary.SWColumnHome" remote="com.supportwizard.dictionary.SWColumn"
 * @jboss:ejb-ref-jndi ref-name="dictionary/SWColumn" jndi-name="dictionary/SWColumn"
 * @ejb:ejb-external-ref ejb-name="dictionary/SWIndex" type="Entity" home="com.supportwizard.dictionary.SWIndexHome" remote="com.supportwizard.dictionary.SWIndex"
 * @jboss:ejb-ref-jndi ref-name="dictionary/SWIndex" jndi-name="dictionary/SWIndex"
 * @ejb:ejb-external-ref ejb-name="dictionary/SWDomain" type="Entity" home="com.supportwizard.dictionary.SWDomainHome" remote="com.supportwizard.dictionary.SWDomain"
 * @jboss:ejb-ref-jndi ref-name="dictionary/SWDomain" jndi-name="dictionary/SWDomain"
 * @ejb:ejb-external-ref ejb-name="dictionary/SWType" type="Entity" home="com.supportwizard.dictionary.SWTypeHome" remote="com.supportwizard.dictionary.SWType"
 * @jboss:ejb-ref-jndi ref-name="dictionary/SWType" jndi-name="dictionary/SWType"
 * @ejb:ejb-external-ref ejb-name="dictionary/SWDictionaryDDL" type="Entity" home="com.supportwizard.dictionary.SWDictionaryDDLHome" remote="com.supportwizard.dictionary.SWDictionaryDDL"
 * @jboss:ejb-ref-jndi ref-name="dictionary/SWDictionaryDDL" jndi-name="dictionary/SWDictionaryDDL"
 * @ejb:ejb-external-ref ejb-name="dml/SWRecord" type="Entity" home="com.supportwizard.dml.SWRecordHome" remote="com.supportwizard.dml.SWRecord"
 * @jboss:ejb-ref-jndi ref-name="dml/SWRecord" jndi-name="dml/SWRecord"
 * @ejb:ejb-external-ref ejb-name="dml/SWData" type="Session" home="com.supportwizard.dml.SWDataHome" remote="com.supportwizard.dml.SWData"
 * @jboss:ejb-ref-jndi ref-name="dml/SWData" jndi-name="dml/SWData"
 * @ejb:ejb-external-ref ejb-name="swusers/SWUsers" type="Session" home="com.supportwizard.swusers.SWUsersHome" remote="com.supportwizard.swusers.SWUsers"
 * @jboss:ejb-ref-jndi ref-name="swusers/SWUsers" jndi-name="swusers/SWUsers"
 */

public abstract class SWAttributeHistoryFunctionalityBean extends SWFunctionalityBase implements javax.ejb.EntityBean {
    private static final Logger log = Logger.getLogger(SWAttributeHistoryFunctionalityBean.class);
    private static final String FUNC_NAME = SWAttributeHistoryFunctionalityConst.FUNC_NAME;
    private static final String FUNC_DESCRIPTION = SWAttributeHistoryFunctionalityConst.FUNC_DESCRIPTION;
    private static final String FUNC_HOME_CLASS_NAME = SWAttributeHistoryFunctionalityConst.FUNC_HOME_CLASS_NAME;
    private static final String FUNC_HOME_JNDI_NAME = SWAttributeHistoryFunctionalityConst.FUNC_HOME_JNDI_NAME;

    private final static String HISTORY_FIELD_TYPE_NAME = SWAttributeHistoryFunctionalityConst.HISTORY_FIELD_TYPE_NAME;
    private final static String HISTORY_FIELD_DOMAIN_NAME = SWAttributeHistoryFunctionalityConst.HISTORY_FIELD_DOMAIN_NAME;

    private String tmpHistoryFieldName; //temp field to pass user defined history name from ejbCreate to getFunctionalityCreationInfo

    /**
     * Name of an attribute that will hold explicitly specified virtual column name. In case of default
     * column name {@link #HISTORY_FIELD_NAME} will be used.
     */
    private static final String HISTORY_FIELD_NAME_ATTR = SWAttributeHistoryFunctionalityConst.HISTORY_FIELD_NAME_ATTR;
    /**
     * Default virtual field name where record history will be inserted. Can be specified
     * using {@link #ejbCreate(Seance, SWTable, String)}.
     */
    private static final String HISTORY_FIELD_NAME = "attribute history";
    /** DB name of virtual field where an attribute history of the record will be inserted.*/
    private static final String HISTORY_FIELD_DB_NAME = "attr_history";

    /**
     * @label logic impl
     */
    private SWAttributeHistoryFunctionalityLogic logic;

    /** @link dependency
     * @stereotype use*/
    /*# AHstTableManager lnkAHstTableManager; */

    /**
     * Creates new attribute history functionality.
     *
     * @param table The SWTable this functionality will be associated with.
     *
     * ejb:create-method
     * ejb:transaction type="Required"
     */
    public Long ejbCreate(SWTable table) throws CreateException {
        throw new CreateException("Please use ejbCreate(Seance, SWTable");
    }

    /**
     * Creates new attribute history functionality.
     *
     * @param table The SWTable this functionality will be associated with.
     *
     * @ejb:create-method
     * @ejb:transaction type="Required"
     */
    public Long ejbCreate(Seance seance, SWTable table) throws CreateException {
        log.info("seance1 = "+seance);
        return ejbCreate(seance, table, null);
    }

    /**
     * Creates new attribute history functionality.
     *
     * @param table The SWTable this functionality will be associated with.
     * @param historyColumnName Virtual history column name (<b>not</b> a dbname).
     *
     * @ejb:create-method
     * @ejb:transaction type="Required"
     */
    public Long ejbCreate(Seance seance, SWTable table, String historyColumnName) throws CreateException {
        this.table = table; //initialization in super.ejbCreate isn't satisfactory coz I need it before that call
        log.info("seance2 = "+seance);
        setHomesGetter(seance.getHomesGetter());
        {
            //this code goes before ejbFindByTable(...) call (12 lines below) because ejbFindByTable
            //indirectly leads to getFunctionalityCreationInfo call via this chain:
            //super.ejbFindByTable->super.getCachedFunctionalityCreationInfo->getFunctionalityCreationInfo
            //and getFunctionalityCreationInfo uses tmpHistoryFieldName value
            if (historyColumnName != null)
                tmpHistoryFieldName = historyColumnName;
            else
                tmpHistoryFieldName = HISTORY_FIELD_NAME;
        }

        //check that it is the only attribute history functionality on this table
        try {
            Collection c = ejbFindByTable(table);
            if (c.size() > 0)
                throw new CreateException("There can be only one attribute history functionality per table");
        } catch (FinderException e) {
            //Not found.
        }
        createAttributeHistoryColumn(seance, table);

        //create _ahst_ table
        AHstTableManagerFactory.getAHstTableManager(null).createHstTable(table);

        return super.ejbCreate(seance, table);
    }

    private void createAttributeHistoryColumn(Seance seance, SWTable table) {
        String name = tmpHistoryFieldName;
        String dbName = HISTORY_FIELD_DB_NAME;
        //create or find the type
        SWType type = getAttrHistoryType(seance);
        //create or find te domain
        Long domainID = getAttrHistoryDomainID(seance, type);
        //now create the column itself
        try {
            SWColumnValue columnValue = new SWColumnValue(name, dbName, domainID, name, name, true, false, "", true);
            SWDictionaryDDLHome dictionaryDDLHome = (SWDictionaryDDLHome) seance.getHome(SWDictionaryDDLHome.class, "dictionary/SWDictionaryDDL");
            SWDictionaryDDL dictionaryDDL = dictionaryDDLHome.create();
            Long projectID = table.getSWProjectID();
            List columns = new ArrayList(1);
            columns.add(columnValue);
            dictionaryDDL.addColumns(projectID.longValue(), table.getName(), columns);
            dictionaryDDL.remove();
        } catch (Exception e) {
            log.debug("Can't create the attribute history column", e);
            throw new EJBException(e);
        }
    }

    private Long getAttrHistoryDomainID(Seance seance, SWType type) {
        SWDomainHome domainHome = getSWDomainHome(seance);
        Long domainID;
        try {
            domainID = findAttrHistoryDomain(domainHome);
        } catch (ObjectNotFoundException e) {
            domainID = createAttrHistoryDomain(domainHome, type);
        } catch (Exception e) {
            log.debug("Exception caught", e);
            throw new EJBException(e);
        }
        return domainID;
    }

    private Long createAttrHistoryDomain(SWDomainHome domainHome, SWType type) {
        Long domainID;
        try {
            SWDomain domain = domainHome.create(HISTORY_FIELD_DOMAIN_NAME, "", "", "", null, type);
            domainID = domain.getSWDomainID();
        } catch (Exception e1) {
            log.error("Can't create domain for the attribute history column");
            throw new EJBException(e1);
        }
        return domainID;
    }

    private Long findAttrHistoryDomain(SWDomainHome domainHome) throws FinderException, RemoteException {
        Long domainID;
        SWDomain domain = domainHome.findByName(HISTORY_FIELD_DOMAIN_NAME);
        domainID = domain.getSWDomainID();
        return domainID;
    }

    private SWType getAttrHistoryType(Seance seance) {
        SWTypeHome typeHome = getSWTypeHome(seance);
        SWType type;
        try {
            type = findAttrHistoryType(typeHome);
        } catch (ObjectNotFoundException e) {
            type = createAttrHistoryType(typeHome);
        } catch (Exception e) {
            if (log.isDebugEnabled())
                log.debug("Exception caught", e);
            throw new EJBException(e);
        }
        return type;
    }

    private SWType createAttrHistoryType(SWTypeHome typeHome) {
        SWType type;
        try {
            SWTypeValue typeValue = new SWTypeValue();
            typeValue.setDataType(Types.LONGVARCHAR);
            typeValue.setNullable(true);
            typeValue.setName(HISTORY_FIELD_TYPE_NAME);
            type = typeHome.create(typeValue);
        } catch (Exception e1) {
            log.error("Can't create type for the attribute history column");
            throw new EJBException(e1);
        }
        return type;
    }

    private SWType findAttrHistoryType(SWTypeHome typeHome) throws FinderException, RemoteException {
        return typeHome.findByName(HISTORY_FIELD_TYPE_NAME);
    }

    private SWDomainHome getSWDomainHome(Seance seance) {
        SWDomainHome domainHome;
        try {
            domainHome = (SWDomainHome) seance.getHome(SWDomainHome.class, "dictionary/SWDomain");
        } catch (NamingException e) {
            log.debug("Exception caught", e);
            throw new EJBException(e);
        }
        return domainHome;
    }

    private SWTypeHome getSWTypeHome(Seance seance) {
        SWTypeHome typeHome;
        try {
            typeHome = (SWTypeHome) seance.getHome(SWTypeHome.class, "dictionary/SWType");
        } catch (NamingException e) {
            log.debug("Exception caught", e);
            throw new EJBException(e);
        }
        return typeHome;
    }

    public void ejbPostCreate(SWTable table) {
    }

    public void ejbPostCreate(Seance seance, SWTable table) {
    }

    public void ejbPostCreate(Seance seance, SWTable table, String historyColumnName) {
    }

    ///////////////////////////////////////////////////////////////////////////
    /**
     * Removes attribute history functionality instance.
     * @ejb:transaction type="Required"
     */
    public void ejbRemove() throws RemoveException {
        //log.debug("Removing attribute history functionality");
        HomesGetter homesGetter;

        setEnabled(false);
        try {
            homesGetter = new HomesGetter();
        } catch (NamingException e) {
            log.error("Error getting homes getter: " + e, e);
            throw new EJBException(e);
        }

        AHstTableManager ahstTableManager = AHstTableManagerFactory.getAHstTableManager(null);

        try {
            ahstTableManager.getAHstTableSize(getSWTableID()); //this will throw an exception if table doesn't exist

            SWAttributeHistoryLogRecordHome historyLogRecordHome = (SWAttributeHistoryLogRecordHome) homesGetter.getHome(
                    SWAttributeHistoryLogRecordHome.class);

            SWAttributeChangeHome attributeChangeHome =
                    (SWAttributeChangeHome) homesGetter.getHome(SWAttributeChangeHome.class);

            Collection c = historyLogRecordHome.findAll(getSWTableID());
            for (Iterator it = c.iterator(); it.hasNext();) {
                SWAttributeHistoryLogRecord logRecord = (SWAttributeHistoryLogRecord) it.next();

                SWAttributeHistoryLogRecordPK pk = (SWAttributeHistoryLogRecordPK) logRecord.getPrimaryKey();
                Long recordID = pk.getRecordID();

                Collection changes = attributeChangeHome.findByRecordID(recordID);
                for (Iterator it2 = changes.iterator(); it2.hasNext();) {
                    SWAttributeChange attributeChange = (SWAttributeChange) it2.next();
                    attributeChange.remove();
                }

                logRecord.remove();
            }

            //now _ahst_ table must be empty
            //but I don't believe myself so I'll check it up
            int size = ahstTableManager.getAHstTableSize(getSWTableID());
            if (size != 0)
                throw new RemoveException("_ahst_ table must be empty! It still contains " + size + " records");
        } catch (EJBException2 e) {
            if (e.getCausedByException() instanceof SW_SQLException)
                log.warn("_ahst_ table doesn exist");
            else {
                log.error("Error clearing _ahst_ table", e);
                throw new RemoveException("Error clearing _ahst_ table, see log for details");
            }
        } catch (RemoteException e) {
            log.error("Communication error", e);
            throw new EJBException(e);
        } catch (FinderException e) {
            log.error("Error clearing _ahst_ table", e);
            throw new RemoveException("Error clearing _ahst_ table, see log for details");
        } catch (NoSuchFieldException e) {
            log.error("Error clearing _ahst_ table", e);
            throw new RemoveException("Error clearing _ahst_ table, see log for details");
        } catch (IllegalAccessException e) {
            log.error("Unexpected exception", e);
            throw new EJBException(e);
        } catch (NamingException e) {
            log.error("Unexpected exception", e);
            throw new EJBException(e);
        }

        //remove artificial history column
        String name;
        try {
            name = getFuncParameterHome().
                    findByPrimaryKey(new SWFuncParameterPK(getFunctionalityID(), HISTORY_FIELD_NAME_ATTR)).
                    getParameterValue();
        } catch (Exception e) {
            log.error("Can't get HISTORY_FIELD_NAME_ATTR parameter", e);
            throw new EJBException(e);
        }
        try {
            Long projectID = table.getSWProjectID();
            String tableName = table.getName();
            SWDictionaryDDLHome dictionaryDDLHome = (SWDictionaryDDLHome) homesGetter.getHome(SWDictionaryDDLHome.class, "dictionary/SWDictionaryDDL");
            SWDictionaryDDL dictionaryDDL = dictionaryDDLHome.create();
            dictionaryDDL.dropColumn(projectID.longValue(), tableName, name);
            dictionaryDDL.remove();
        } catch (Exception e) {
            log.error("Can't remove artificial history column", e);
        }

        ahstTableManager.dropAHstTable(getSWTableID());
        super.ejbRemove();
    }

    ///////////////////////////////////////////////////////////////////////////
    /**
     * Gets functionality logic implementation.
     *
     * @return Functionality logic instance.
     *
     * ejb:interface-method view-type="remote"
     */
    public SWFunctionalityLogic getFunctionalityLogic() {
        if (logic != null)
            return logic;
        logic = new SWAttributeHistoryFunctionalityLogic(getFunctionalityID(), getSWTableID());
        return logic;
    }

    //////////////////////////////////////////////////////////////////////////
    /**
     * Gets functionality creation info.
     *
     * @return Functionality creation info.
     */
    protected SWFunctionalityCreationInfo getFunctionalityCreationInfo() {
        SWFunctionalityCreationInfo res = new SWFunctionalityCreationInfo();
        res.funcName = FUNC_NAME;
        res.funcDescription = FUNC_DESCRIPTION;
        res.funcHomeClassName = FUNC_HOME_CLASS_NAME;
        res.funcHomeJndiName = FUNC_HOME_JNDI_NAME;
        res.addParameterCreationInfo(new SWFuncParameterCreationInfo(
                HISTORY_FIELD_NAME_ATTR, "Artificial history column name", tmpHistoryFieldName));
        return res;
    }
}
