/*
 * SupportWizard 2
 * 
 * Copyright (C) 2002 ISC corp. All Rights Reserved.
 * 
 * $Id: SWAttributeHistoryLogRecordBean.java,v 1.11 2003/07/22 23:05:31 alexi Exp $
 * Created by Konstantin Sobolev (kos@supportwizard.com) on 17.06.2002
 * Last modification $Date: 2003/07/22 23:05:31 $
 */
package com.supportwizard.functionalities.attributehistory.ejb;

import com.supportwizard.db.ColumnList;
import com.supportwizard.db.ConditionList;
import com.supportwizard.db.SWInternal_SQL;
import com.supportwizard.db.TableList;
import com.supportwizard.dictionary.util.ColumnNamesHelper;
import com.supportwizard.dml.SWRecordPK;
import com.supportwizard.functionalities.attributehistory.AHstTableManagerFactory;
import com.supportwizard.functionalities.attributehistory.interfaces.SWAttributeChange;
import com.supportwizard.functionalities.attributehistory.interfaces.SWAttributeChangeHome;
import com.supportwizard.functionalities.attributehistory.interfaces.SWAttributeHistoryLogRecordPK;
import com.supportwizard.utils.EJBException2;
import com.supportwizard.utils.HomesGetter;
import org.apache.log4j.Logger;
import javax.ejb.*;
import java.rmi.RemoteException;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.*;

/**
 * This entity bean describes one modification of one or many attributes performed as an integral action
 * by some person. Single attribute chhangements are realized by <code>SWAttributeChange</code> bean.
 * <p>
 * All relevant information is stored in so-called _ahst_ table. When new instance of attribute history
 * functionality appears it creates new _ahst_ table with name <target SWTable ID>_ahst_, so if target
 * <code>SWTable</code>'s ID is <code>17</code> then _ahst_ table will be named 17_ahst_. The suffix is stored
 * in {@link #ATTR_HISTORY_TABLE_NAME_SUFFIX ATTR_HISTORY_TABLE_NAME_SUFFIX} variable.
 * <p>
 * See {@link com.supportwizard.functionalities.attributehistory.AHstTableManager AHstTableManager} for more details
 * on _ahst_ tables.
 * 
 * @author <a href="mailto:kos@supportwizard.com" title="">Konstantin Sobolev</a>
 * @version $Revision: 1.11 $
 * 
 * @ejb:bean name="SWAttributeHistoryLogRecord" type="BMP" jndi-name="functionalities/attributehistory/SWAttributeHistoryLogRecord" transaction-type="Container"
 * @ejb:transaction type="Supports"
 * @ejb:data-object container="false"
 * @ejb:finder signature="java.util.Collection findBySWRecordPK(com.supportwizard.dml.SWRecordPK recordPK)"
 * @ejb:finder signature="java.util.Collection findAll(Long tableID)"
 * 
 * @ejb:ejb-ref ejb-name="SWAttributeChange" ref-name="functionalities/attributehistory/SWAttributeChange"
 * 
 * @ejb:ejb-external-ref ejb-name="dictionary/SWTable" type="Entity" home="com.supportwizard.dictionary.SWTableHome" remote="com.supportwizard.dictionary.SWTable"
 * @jboss:ejb-ref-jndi ref-name="dictionary/SWTable" jndi-name="dictionary/SWTable"
 * @ejb:ejb-external-ref ejb-name="dictionary/SWDictionaryDDL" type="Session" home="com.supportwizard.dictionary.SWDictionaryDDLHome" remote="com.supportwizard.dictionary.SWDictionaryDDL"
 * @jboss:ejb-ref-jndi ref-name="dictionary/SWDictionaryDDL" jndi-name="dictionary/SWDictionaryDDL"
 * 
 * @ejb:resource-ref res-name="jdbc/sw" res-type="javax.sql.DataSource" res-auth="Container"
 */
public abstract class SWAttributeHistoryLogRecordBean implements EntityBean {
  private final static Logger log = Logger.getLogger(SWAttributeHistoryLogRecordBean.class);

  /**
   * Suffix for table name that will hold history log.
   */
  private static final String ATTR_HISTORY_TABLE_NAME_SUFFIX = 
    SWAttributeHistoryLogRecordConst.ATTR_HISTORY_TABLE_NAME_SUFFIX;

  /**
   * Suffix for field names that will hold primary key field values
   */
  private static final String PK_FIELD_NAMES_SUFFIX = SWAttributeHistoryLogRecordConst.PK_FIELD_NAMES_SUFFIX;
  private EntityContext       entityContext;
  private String              ahstTableName;

  /**
   * @link dependency
   * @label describes
   */

  /* # SWAttributeChangeBean lnkSWAttributeChangeBean; */

  /**
   * Gets table ID. Table ID isn't stored into the DB, only in the PK class for techincal purposes.
   * 
   * @return Table ID.
   * 
   * @ejb:persistent-field
   * @ejb:pk-field
   */
  public abstract Long getTableID();

  /**
   * Sets table ID.
   * 
   * @param tableID Table ID.
   * 
   * @ejb:persistent-field
   * @ejb:pk-field
   */
  public abstract void setTableID(Long tableID);

  /**
   * Gets history log record's ID.
   * 
   * @return Record ID.
   * 
   * @ejb:pk-field
   * @ejb:persistent-field
   */
  public abstract Long getRecordID();

  /**
   * Sets history log record's ID.
   * 
   * @param recordID new record ID.
   */
  public abstract void setRecordID(Long recordID);

  /**
   * Gets action date in milliseconds.
   * 
   * @return Action date.
   * 
   * @ejb:persistent-field
   * @ejb:interface-method
   */
  public abstract Long getActionDate();

  /**
   * Sets action date in milliseconds.
   * 
   * @param actionDate new action date in milliseconds.
   */
  public abstract void setActionDate(Long actionDate);

  /**
   * Gets ID of a user who performed an action.
   * 
   * @return User ID.
   * 
   * @ejb:persistent-field
   * @ejb:interface-method
   */
  public abstract Long getUserID();

  /**
   * Sets ID of a user who performed an action.
   * 
   * @param userID new user ID.
   */
  public abstract void setUserID(Long userID);

  /**
   * Gets primary key of a user record whose attributes have been changed.
   * User record is a record from <code>SWTable</code> attribute history functionality instance is attached to.
   * 
   * @return Primary key of a user record whose attributes have been changed.
   * 
   * @ejb:persistent-field
   * @ejb:interface-method
   */
  public abstract SWRecordPK getSWRecordPK();

  /**
   * Sets primary key of a user record whose attributes have been changed.
   * User record is a record from <code>SWTable</code> attribute history functionality instance is attached to.
   * 
   * @param recordPK primary key of a user record whose attributes have been changed.
   */
  public abstract void setSWRecordPK(SWRecordPK recordPK);

  // ______________________________________________

  /**
   * Creates new <code>SWAttributeHistoryLogRecord</code> instance.
   * 
   * @param userTableRecordPK Primary key of a user record whose attributes have been changed.
   * @param actionDate Action date in milliseconds.
   * @param userID ID of a user who performed an action.
   * 
   * @return Primary key of the created instance.
   * 
   * @throws CreateException if some application-level error occurrs.
   * 
   * @ejb:create-method
   * @ejb:transaction type="Required"
   */
  public SWAttributeHistoryLogRecordPK ejbCreate(SWRecordPK userTableRecordPK, Long actionDate, 
          Long userID) throws CreateException {
    setSWRecordPK(userTableRecordPK);
    setTableID(userTableRecordPK.getTableID());
    setActionDate(actionDate);
    setUserID(userID);

    SWInternal_SQL sw_sql = null;

    if (log.isDebugEnabled()) {
      log.debug("Creating SWAttributeHistoryLogRecord with actionDate=" + actionDate + ", userID=" + userID 
                + ", SWRecordPK=" + userTableRecordPK);
    } 

    SWAttributeHistoryLogRecordPK pk = null;

    try {
      sw_sql = new SWInternal_SQL();

      sw_sql.dbConnect();
    } catch (Throwable e) {
      log.error("Can't connect to the database", e);

      throw new EJBException2(e, "Can't connect to the database");
    } 

    try {
      Long tableID = userTableRecordPK.getTableID();

      ahstTableName = getAHstTableName(tableID);

      String     tableName = ahstTableName;
      Map        columnNames2dbColumnNames = new ColumnNamesHelper().getNamesToDBNamesMapping(tableID);
      ColumnList columnList = new ColumnList();

      columnList.add("ActionDate");
      columnList.add("UserID");

      ArrayList values = new ArrayList();

      values.add(actionDate);
      values.add(userID);

      Map       pkFields = userTableRecordPK.getPKFields();
      ArrayList pkFieldNames = new ArrayList();
      Iterator  it = pkFields.keySet().iterator();

      while (it.hasNext()) {
        String name = (String) it.next();

        pkFieldNames.add(name);
        columnList.add(columnNames2dbColumnNames.get(name) + PK_FIELD_NAMES_SUFFIX);
        values.add(pkFields.get(name));
      } 

      HashMap params = new HashMap(3);

      params.put("table", tableName);
      params.put("columns", columnList);
      params.put("values", values);

      if (sw_sql.sqlDML("insert", params) == 0) {
        throw new CreateException("Can't create new SWAttributeHistoryLogRecord instance");
      } 

      Long recordID = new Long(sw_sql.getInsertID());

      setRecordID(recordID);

      pk = new SWAttributeHistoryLogRecordPK(tableID, recordID);

      if (log.isDebugEnabled()) {
        log.debug("Created SWAttributeHistoryLogRecord with pk=" + pk);
      } 
    } catch (CreateException e) {
      log.debug("Exception caught", e);

      throw e;
    } catch (Throwable e) {
      log.debug("Exception caught while creating SWAttributeHistoryLogRecord with pk=" + pk, e);

      throw new EJBException2(e);
    } 
    finally {
      try {
        sw_sql.dbDisconnect();
      } catch (SQLException e) {
        log.error("Cant't close database connection", e);
      } 
    } 

    return pk;
  } 

  public void ejbPostCreate(SWRecordPK userTableRecordPK, Long actionDate, Long userID) {}

  /**
   * Removes <code>SWAttributeHistoryLogRecord</code> instance.
   * 
   * @throws RemoveException if some application-level error occurrs.
   * @ejb:transaction type="Required"
   */
  public void ejbRemove() throws RemoveException {
    if (log.isDebugEnabled()) {
      log.debug("Removing SWAttributeHistoryLogRecord with RecordID = " + getRecordID());

      // first kill all attribute change records
    } 

    try {
      HomesGetter           homesGetter = new HomesGetter();
      SWAttributeChangeHome attributeChangeHome = (SWAttributeChangeHome) homesGetter.getHome(SWAttributeChangeHome.class);
      Collection            c = attributeChangeHome.findByRecordID(getRecordID());

      for (Iterator it = c.iterator(); it.hasNext(); ) {
        SWAttributeChange attributeChange = (SWAttributeChange) it.next();

        attributeChange.remove();
      } 
    } catch (Exception e) {
      log.error("Exception while removing attribute changes; skipping", e);
    } 

    SWInternal_SQL sw_sql = null;

    try {
      sw_sql = new SWInternal_SQL();

      sw_sql.dbConnect();
    } catch (Throwable e) {
      log.error("Can't connect to the database", e);

      throw new EJBException("Can't connect to the database");
    } 

    try {
      ConditionList where = new ConditionList();

      where.add("", "RecordID", "", "=", getRecordID(), "");

      HashMap params = new HashMap(2);

      params.put("table", ahstTableName);
      params.put("where", where);

      if (sw_sql.sqlDML("delete", params) == 0) {
        throw new Exception("Cant remove record SWAttributeHistoryLogRecord with RecordID = " + getRecordID());
      } 
    } catch (Throwable e) {
      log.error("Exception caught", e);

      throw new EJBException2(e);
    } 
    finally {
      try {
        sw_sql.dbDisconnect();
      } catch (SQLException e) {
        log.error("Cant't close database connection", e);

        // throw new RemoveException(e.getMessage());
      } 
    } 
  } 

  public void setEntityContext(EntityContext context) {
    entityContext = context;
  } 

  public void ejbLoad() {
    SWInternal_SQL                sw_sql;
    SWAttributeHistoryLogRecordPK pk = (SWAttributeHistoryLogRecordPK) entityContext.getPrimaryKey();

    setRecordID(pk.getRecordID());
    setTableID(pk.getTableID());

    try {
      sw_sql = new SWInternal_SQL();

      sw_sql.dbConnect();
    } catch (Throwable e) {
      log.error("Can't connect to the database", e);

      throw new EJBException2(e, "Can't connect to the database");
    } 

    try {
      if (ahstTableName == null) {
        ahstTableName = getAHstTableName(getTableID());
      } 

      ColumnList    columnList = new ColumnList();  // select all columns
      ConditionList where = new ConditionList();

      where.add("", "RecordID", "", "=", getRecordID(), "");

      TableList from = new TableList();

      from.add(ahstTableName, "");

      HashMap params = new HashMap(3);

      params.put("from", from);
      params.put("columns", columnList);
      params.put("where", where);

      ResultSet rs = sw_sql.sqlQuery(params);

      if (!rs.next()) {
        rs.close();

        throw new EJBException("Can't find SWAttributeHistoryLogRecord with pk=" + pk);
      } 

      setActionDate(new Long(rs.getLong("ActionDate")));
      setUserID(new Long(rs.getLong("UserID")));

      Map               dbColumnNames2ColumnNames = new ColumnNamesHelper().getDBNamesToNamesMapping(getTableID());
      ResultSetMetaData metaData = rs.getMetaData();
      int               columnsCount = metaData.getColumnCount();
      Map               pkFields = new HashMap();

      for (int i = 1; i <= columnsCount; i++) {
        String columnName = metaData.getColumnName(i);

        if (!columnName.endsWith(PK_FIELD_NAMES_SUFFIX)) {
          continue;

          // cut suffix
        } 

        String _columnName = columnName.substring(0, columnName.length() - PK_FIELD_NAMES_SUFFIX.length());

        // convert from db name to user name and save
        pkFields.put(dbColumnNames2ColumnNames.get(_columnName), rs.getObject(i));
      } 

      setSWRecordPK(new SWRecordPK(getTableID(), pkFields));
      rs.close();
    } catch (Throwable e) {
      log.error("Exception caught", e);

      throw new EJBException2(e);
    } 
    finally {
      try {
        sw_sql.dbDisconnect();
      } catch (SQLException e) {
        log.error("Cant't close database connection", e);
      } 
    } 
  } 

  // _______________________finder methods_______________________

  /**
   * Checks if there exists a <code>SWAttributeHistoryLogRecord</code> instance with given primary key.
   * 
   * @param key Primary key of the instance we're looking for.
   * @return Primary key of the found instance.
   * 
   * @throws ObjectNotFoundException if instance with this primary key doesn't exist.
   */
  public SWAttributeHistoryLogRecordPK ejbFindByPrimaryKey(SWAttributeHistoryLogRecordPK key) throws FinderException {
    SWInternal_SQL sw_sql = null;

    try {
      sw_sql = new SWInternal_SQL();

      sw_sql.dbConnect();
    } catch (Throwable e) {
      log.error("Can't connect to the database", e);

      throw new EJBException2(e, "Can't connect to the database");
    } 

    try {
      ColumnList columnList = new ColumnList();

      columnList.add("RecordID");

      TableList from = new TableList();

      // from.add(getAHstTableName(getSWRecordPK().getTableID()), "");
      from.add(getAHstTableName(key.getTableID()), "");

      ConditionList where = new ConditionList();

      where.add("", "RecordID", "", "=", key.getRecordID(), "");

      HashMap params = new HashMap(3);

      params.put("from", from);
      params.put("columns", columnList);
      params.put("where", where);

      ResultSet rs = sw_sql.sqlQuery(params);

      if (!rs.next()) {
        rs.close();

        throw new ObjectNotFoundException("Cant find SWAttributeHistoryLogRecord with RecordID = " + key.getRecordID());
      } 

      Long recID = new Long(rs.getLong(1));

      key = new SWAttributeHistoryLogRecordPK(key.getTableID(), recID);

      rs.close();
    } catch (FinderException e) {
      log.debug("Exception caught", e);

      throw e;
    } catch (Throwable e) {
      log.debug("Exception caught", e);

      throw new EJBException2(e);
    } 
    finally {
      try {
        sw_sql.dbDisconnect();
      } catch (SQLException e) {
        log.error("Cant't close database connection", e);
      } 
    } 

    return key;
  } 

  /**
   * Finds all history records that describe changes of the attributes of the specified record. Results are
   * ordered by time, i.e. oldest actions go first.
   * 
   * @param recordPK Target <code>SWRecord</code> primary key.
   * @return Collection of found primary keys.
   * 
   * @throws FinderException if application-level error occurs.
   */
  public Collection ejbFindBySWRecordPK(SWRecordPK recordPK) throws FinderException {
    SWInternal_SQL sw_sql = null;

    try {
      sw_sql = new SWInternal_SQL();

      sw_sql.dbConnect();
    } catch (Throwable e) {
      log.error("Can't connect to the database", e);

      throw new EJBException2(e, "Can't connect to the database");
    } 

    try {
      ColumnList columnList = new ColumnList();

      columnList.add("RecordID");

      ColumnList order = new ColumnList();

      order.add("RecordID");

      Long          tableID = recordPK.getTableID();
      ConditionList where = new ConditionList();

      // fill primary key of user table
      Map           pkFields = recordPK.getPKFields();
      Map           columnNames2dbColumnNames = new ColumnNamesHelper().getNamesToDBNamesMapping(recordPK.getTableID());
      Iterator      it = pkFields.keySet().iterator();
      boolean       first = true;

      // StringBuffer whereClause = new StringBuffer();
      while (it.hasNext()) {
        String name = (String) it.next();
        String dbName = ((String) columnNames2dbColumnNames.get(name));

        if (dbName == null) {
          log.warn("Can't find DBName for column '" + name + "' of table with ID=" + tableID);

          dbName = name;
        } 

        Object val = pkFields.get(name);

        where.add("", dbName + PK_FIELD_NAMES_SUFFIX, "", "=", val, first ? "" : "AND");

        // whereClause.append(first?"":" AND ").append(dbName).append(PK_FIELD_NAMES_SUFFIX).append("=").append(val);
        first = false;
      } 

      // log.debug("where = " + whereClause);
      TableList from = new TableList();

      from.add(getAHstTableName(tableID), "");

      HashMap params = new HashMap(4);

      params.put("from", from);
      params.put("columns", columnList);
      params.put("where", where);
      params.put("order", order);

      ResultSet  rs = sw_sql.sqlQuery(params);
      Collection res = new ArrayList();

      while (rs.next()) {
        res.add(new SWAttributeHistoryLogRecordPK(tableID, new Long(rs.getLong(1))));
      } 

      rs.close();

      return res;
    } catch (Throwable e) {
      log.debug("Exception caught", e);

      throw new EJBException2(e);
    } 
    finally {
      try {
        sw_sql.dbDisconnect();
      } catch (SQLException e) {
        log.error("Cant't close database connection", e);
      } 
    } 
  } 

  /**
   * Finds all instances for the given <code>SWTable</code>. Actually returns <em>all</em> instances
   * as far as all of them belong to the same functionality and thus to the same table. <code>tableID</code>
   * used for technical simplicity only.
   * 
   * @param tableID <code>SWTable</code>'s ID.
   * @return Collection of found primary keys.
   * 
   * @throws FinderException if application-level error occurs.
   */
  public Collection ejbFindAll(Long tableID) throws FinderException {
    SWInternal_SQL sw_sql = null;

    try {
      sw_sql = new SWInternal_SQL();

      sw_sql.dbConnect();
    } catch (Throwable e) {
      log.error("Can't connect to the database", e);

      throw new EJBException2(e, "Can't connect to the database");
    } 

    try {
      ColumnList columnList = new ColumnList();

      columnList.add("RecordID");

      ColumnList order = new ColumnList();

      order.add("RecordID");

      ConditionList where = new ConditionList();
      TableList     from = new TableList();

      from.add(getAHstTableName(tableID), "");

      HashMap params = new HashMap(4);

      params.put("from", from);
      params.put("columns", columnList);
      params.put("where", where);
      params.put("order", order);

      ResultSet  rs = sw_sql.sqlQuery(params);
      Collection res = new ArrayList();

      while (rs.next()) {
        res.add(new SWAttributeHistoryLogRecordPK(tableID, new Long(rs.getLong(1))));
      } 

      rs.close();

      return res;
    } catch (Throwable e) {
      log.debug("Exception caught", e);

      throw new EJBException2(e);
    } 
    finally {
      try {
        sw_sql.dbDisconnect();
      } catch (SQLException e) {
        log.error("Cant't close database connection", e);
      } 
    } 
  } 

  // _______________________private area_______________________

  /**
   * Gets _ahst_ table name.
   * 
   * @param tableID ID of target user table.
   * @return _ahst_ table name.
   */
  public String getAHstTableName(Long tableID) throws RemoteException {
    return AHstTableManagerFactory.getAHstTableManager(null).getAHstTableName(tableID);
  } 

}

