/*
 * SupportWizard 2
 * 
 * Copyright (C) 2002 ISC corp. All Rights Reserved.
 * 
 * $Id: SWAttributeHistoryWriterBean.java,v 1.8 2003/07/22 23:05:31 alexi Exp $
 * Created by Konstantin Sobolev (kos@supportwizard.com) on 19.06.2002
 * Last modification $Date: 2003/07/22 23:05:31 $
 */
package com.supportwizard.functionalities.attributehistory.ejb;

import org.apache.log4j.Category;
import javax.ejb.SessionBean;
import javax.ejb.CreateException;
import javax.ejb.ObjectNotFoundException;
import com.supportwizard.seance.Seance;
import com.supportwizard.utils.EJBException2;
import com.supportwizard.utils.HomesGetter;
import com.supportwizard.functionalities.attributehistory.interfaces.*;
import com.supportwizard.dml.SWRecordPK;
import java.util.Date;
import java.util.List;
import java.rmi.RemoteException;

/**
 * Attribute history writer. Allows easy attribute history updating.
 * See <a href="../package-summary.html">package documentation</a> for the usage example.
 * 
 * @author <a href="mailto:kos@supportwizard.com" title="">Konstantin Sobolev</a>
 * @version $Revision: 1.8 $
 * 
 * @ejb:bean name="SWAttributeHistoryWriter" type="Stateless" jndi-name="functionalities/attributehistory/SWAttributeHistoryWriter" transaction-type="Container"
 * @ejb:transaction type="Supports"
 * @ejb:ejb-ref ejb-name="SWAttributeChange" ref-name="functionalities/attributehistory/SWAttributeChange"
 * @ejb:ejb-ref ejb-name="SWAttributeHistoryLogRecord"
 * 
 */
public abstract class SWAttributeHistoryWriterBean implements SessionBean {
  private Category cat = Category.getInstance(SWAttributeHistoryWriterBean.class);

  /**
   * @link dependency
   * @stereotype instantiate
   */

  /* # SWAttributeHistoryLogRecordBean lnkSWAttributeHistoryLogRecordBean; */

  /**
   * @link dependency
   * @stereotype instantiate
   */

  /* # SWAttributeChangeBean lnkSWAttributeChangeBean; */

  /**
   * Creates new <code>SWAttributeHistoryWriter</code> instance.
   * 
   * @ejb:create-method
   * @ejb:transaction type="Required"
   */
  public void ejbCreate() throws CreateException {}

  /**
   * Adds an attribute history data.
   * 
   * @param seance Seance.
   * @param swRecordPK Primary key of the user record changed attributes belong to.
   * Table ID will be obtained using <code>getTableID()</code> method.
   * @param userID ID of a user who changed the attributes.
   * @param actionDate Action date.
   * @param attrNames Attribute names.
   * @param oldValues Old attrobute values.
   * @param newValues New attribute values.
   * 
   * @ejb:interface-method view-type="remote"
   * @ejb:transaction type="Required"
   */
  public void addHistoryData(Seance seance, SWRecordPK swRecordPK, Long userID, Date actionDate, List attrNames, 
                             List oldValues, List newValues) {
    HomesGetter homesGetter = seance.getHomesGetter();

    // get homes
    try {
      SWAttributeHistoryLogRecordHome attributeHistoryLogRecordHome = 
        (SWAttributeHistoryLogRecordHome) homesGetter.getHome(SWAttributeHistoryLogRecordHome.class);
      SWAttributeChangeHome           attributeChangeHome = 
        (SWAttributeChangeHome) homesGetter.getHome(SWAttributeChangeHome.class);

      // do the work
      SWAttributeHistoryLogRecord     attributeHistoryLogRecord = attributeHistoryLogRecordHome.create(swRecordPK, 
              new Long(actionDate.getTime()), userID);
      SWAttributeHistoryLogRecordPK   pk = (SWAttributeHistoryLogRecordPK) attributeHistoryLogRecord.getPrimaryKey();
      Long                            recordID = pk.getRecordID();

      for (int i = 0; i < attrNames.size(); i++) {
        String attrName = (String) attrNames.get(i);
        String oldValue = (String) oldValues.get(i);
        String newValue = (String) newValues.get(i);

        // remove old copy
        try {
          SWAttributeChange attributeChange = attributeChangeHome.findByPrimaryKey(new SWAttributeChangePK(recordID, 
                  attrName));

          attributeChange.remove();
        } catch (ObjectNotFoundException e) {}

        attributeChangeHome.create(recordID, attrName, oldValue, newValue);
      } 
    } catch (Throwable e) {
      cat.error("Error adding attribute history data", e);

      throw new EJBException2(e, "Error adding attribute history data");
    } 
  } 

  /**
   * Adds attribute history data. User ID is taken from seance. Uses current system time as an action date.
   * 
   * @param seance Seance.
   * @param swRecordPK Primary key of the user record changed attributes belong to.
   * Table ID will be obtained using <code>getTableID()</code> method.
   * @param attrNames Attribute names.
   * @param oldValues Old attrobute values.
   * @param newValues New attribute values.
   * 
   * @ejb:interface-method view-type="remote"
   * @ejb:transaction type="Required"
   */
  public void addHistoryData(Seance seance, SWRecordPK swRecordPK, List attrNames, List oldValues, List newValues) {
    Long userID = null;

    try {
      userID = seance.getUserID();
    } catch (RemoteException e) {
      cat.error("Remote exception while getting user id", e);

      throw new EJBException2(e, "Remote exception while getting user id");
    } 

    Date actionDate = new Date(System.currentTimeMillis());

    addHistoryData(seance, swRecordPK, userID, actionDate, attrNames, oldValues, newValues);
  } 

}

