/*
 * SupportWizard 2
 * 
 * Copyright (C) 2001 ISC corp. All Rights Reserved.
 * 
 * $Id: SWBlobFunctionalityBean.java,v 1.18 2003/07/22 23:05:31 alexi Exp $
 */
package com.supportwizard.functionalities.blob.ejb;

import java.rmi.RemoteException;
import java.util.Iterator;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.ArrayList;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.naming.NamingException;
import com.supportwizard.dictionary.SWColumnValue;
import com.supportwizard.dictionary.SWDictionaryDDL;
import com.supportwizard.dictionary.SWDictionaryDDLHome;
import com.supportwizard.dictionary.SWTable;
import com.supportwizard.functionalities.SWFunctionalityBase;
import com.supportwizard.functionalities.SWFunctionalityException;
import com.supportwizard.functionalities.SWFunctionalityLogic;
import com.supportwizard.functionalities.SWFunctionalityCreationInfo;
import com.supportwizard.functionalities.SWFuncParameterCreationInfo;
import com.supportwizard.functionalities.blob.interfaces.SWBlobRecord;
import com.supportwizard.functionalities.blob.interfaces.SWBlobRecordHome;
import com.supportwizard.functionalities.blob.interfaces.SWBlobRecordPK;
import com.supportwizard.utils.HomesGetter;
import org.apache.log4j.Category;

/**
 * 
 * BLOB functionality. It provides the management (adding, removing and modifying) of attachments for SW2 user tables. Each user table can register at most one BLOB functionality. At that, single BLOB functionality allows presence of arbitrary number of attachments for user table.<p>
 * All the attacments for each user table are stored separately in auxiliary database table. BLOB functionality creates this auxiliary table named <code>user_table_id<strong>_blobs</strong></code> during its creation for the given user table. This auxiliary table manages all the attachments of the main table, and is being removed while deletion of BLOB functionality. There is an entity bean {@link com.supportwizard.functionalities.blob.ejb.SWBlobRecordBean SWBlobRecordBean} that manages the worh with attachment tables.<p>
 * Note that users can not add or remove this functionality. It can only be added or removed automatically by the system.
 * @ejb:bean name="SWBlobFunctionality" type="BMP" jndi-name="functionalities/SWBlobFunctionality" transaction-type="Container"
 * @ejb:data-object container="false"
 * @ejb:pk class="java.lang.Long"
 * @ejb:interface extends="javax.ejb.EJBObject, com.supportwizard.functionalities.SWFunctionality"
 * @ejb:resource-ref res-name="jdbc/sw" res-type="javax.sql.DataSource" res-auth="Container"
 * 
 * @ejb:ejb-external-ref ejb-name="dictionary/SWDictionaryDDL" type="Session" home="com.supportwizard.dictionary.SWDictionaryDDLHome" remote="com.supportwizard.dictionary.SWDictionaryDDL"
 * @jboss:ejb-ref-jndi ref-name="dictionary/SWDictionaryDDL" jndi-name="dictionary/SWDictionaryDDL"
 * 
 * @ejb:ejb-external-ref ejb-name="dictionary/SWTable" type="Entity" home="com.supportwizard.dictionary.SWTableHome" remote="com.supportwizard.dictionary.SWTable"
 * @jboss:ejb-ref-jndi ref-name="dictionary/SWTable" jndi-name="dictionary/SWTable"
 * 
 * @ejb:ejb-ref ejb-name="SWBlobRecord" ref-name="functionalities/SWBlobRecord"
 * @author Michael Degtyarev
 * @version $Revision: 1.18 $
 * @see SWBlobRecordBean
 */
public abstract class SWBlobFunctionalityBean extends SWFunctionalityBase implements javax.ejb.EntityBean {
  private transient Category  cat = Category.getInstance(this.getClass().getName());
  private static final String FUNC_NAME = "Blob";
  private static final String FUNC_DESCRIPTION = "Binary large objects functionality";
  private static final String FUNC_HOME_CLASS_NAME = 
    "com.supportwizard.functionalities.blob.interfaces.SWBlobFunctionalityHome";
  private static final String FUNC_HOME_JNDI_NAME = "functionalities/SWBlobFunctionality";
  public final static String  BLOB_TABLE_NAME_SUFFIX = "_blobs";
  public final static String  BLOB_FILENAME_SUFFIX = "_filename";
  private HomesGetter         homesGetter;

  /**
   * Creates new BLOB functionality and registers it for the table passed as parameter. While creating, creates an auxiliary database table named <code>table_id<strong>_blobs</strong></code> as the storage for attachments.
   * 
   * @param table The SWTable this functionality will be associated with.
   * 
   * ejb:create-method
   */

  // bedal commented because SWFucntionalityBase set this already * @ejb:create-method
  public Long ejbCreate(SWTable table) throws CreateException {
    cat.debug("ENTERING ejbCreate(" + table.toString() + ")");

    try {
      this.table = table;

      // At most one BLOB functionality per table can exist
      try {
        Collection col = ejbFindByTable(table);

        if (col.size() != 0) {
          throw new CreateException("At most one BLOB functionality per table can exist.");
        } 
      } catch (FinderException ex) {
        cat.warn("Finder complains:" + ex.getMessage());
      } 

      SWColumnValue blobColumn_1 = new SWColumnValue("blob_id", "blob_id", "SWAutoPK", "blob_id", 
              "Auto-increment primary key for auxiliary attachments table", false, true, null, false, false);
      SWColumnValue blobColumn_2 = new SWColumnValue("blob_name", "blob_name", "SWVarchar", "blob_name", 
              "This field stores attachment file name", false, false, null);
      SWColumnValue blobColumn_3 = new SWColumnValue("blob_content", "blob_content", "SWBlob", "blob_content", 
              "This field stores attachment content", false, false, null);
      ArrayList     columns = new ArrayList();

      columns.add(blobColumn_1);
      columns.add(blobColumn_2);
      columns.add(blobColumn_3);

      SWDictionaryDDLHome ddlHome = (SWDictionaryDDLHome) getHomesGetter().getHome(SWDictionaryDDLHome.class, 
              "dictionary/SWDictionaryDDL");
      SWDictionaryDDL     ddl = ddlHome.create();

      ddl.createTable(table.getSWProjectID().longValue(), getBlobTableName(), "The auxiliary attachments table", 
                      "NOT_FUNCTIONAL", columns, null);

      return super.ejbCreate(table);
    } catch (CreateException ex) {
      cat.error(ex);

      throw ex;
    } catch (Exception ex) {
      cat.error(ex);

      throw new EJBException(ex);
    } 
  } 

  /**
   * Removes this functionality. While removing, deletes the auxiliary attachments table from the database. Before removing this table, removes all the instances of {@link com.supportwizard.functionalities.blob.ejb.SWBlobRecordBean SWBlobRecordBean} entity.
   * @see SWBlobRecordBean
   */
  public void ejbRemove() throws RemoveException {
    cat.debug("ENTERING ejbRemove");
    super.ejbRemove();

    try {

      // First we have to remove all the SWBlobRecordBean's associated with this functionality
      Collection blobRecords = ((SWBlobRecordHome) getHomesGetter().getHome(SWBlobRecordHome.class)).findByTable(table);

      for (Iterator it = blobRecords.iterator(); it.hasNext(); ) {
        SWBlobRecord blobRecord = (SWBlobRecord) it.next();

        blobRecord.remove();
      } 

      SWDictionaryDDLHome ddlHome = (SWDictionaryDDLHome) getHomesGetter().getHome(SWDictionaryDDLHome.class, 
              "dictionary/SWDictionaryDDL");
      SWDictionaryDDL     ddl = ddlHome.create();

      ddl.dropTable(table.getSWProjectID().longValue(), getBlobTableName());
    } catch (Exception ex) {
      cat.error(ex);

      throw new RemoveException(ex.getMessage());
    } 
  } 

  public void ejbPassivate() {}

  public void ejbActivate() throws EJBException {}

  // /////////////////////////////////////////////////////////////////////////

  /**
   * Instantiates and returns the instance of <code>SWBlobFunctionalityLogic</code> for this functionality.
   * @return <code>SWBlobFunctionalityLogic</code> instance.
   * 
   * ejb:interface-method view-type="remote"
   * @see SWBlobFunctionalityLogic
   */

  // bedal commented because SWFucntionalityBase set this already
  public SWFunctionalityLogic getFunctionalityLogic() {
    try {
      return new SWBlobFunctionalityLogic(super.getFunctionalityID(), super.getSWTableID());
    } catch (Exception ex) {
      cat.error("Exception caught", ex);

      throw new EJBException("Can't create SWBlobFunctionalityLogic, see log for details");
    } 
  } 

  /**
   * Returns the content of the BLOB for specified blob ID.
   * @ejb:interface-method view-type="remote"
   * @param blobID An identifier (PK) of the BLOB to return the content.
   * @return BLOB content
   */
  public Object getBlobContent(Integer blobID) throws SWFunctionalityException {
    cat.debug("ENTERING getBlobContent(" + blobID + ")");

    SWBlobRecordPK blobRecordPK = new SWBlobRecordPK(super.getSWTableID(), blobID);

    try {
      SWBlobRecord blobRecord = 
        ((SWBlobRecordHome) getHomesGetter().getHome(SWBlobRecordHome.class)).findByPrimaryKey(blobRecordPK);
      Object       blobContent = blobRecord.getBlobContent();

      return blobContent;
    } catch (NamingException ex) {
      cat.error("Failed to resolve JNDI name for SWBlobRecordHome home interface", ex);

      throw new SWFunctionalityException(ex.getMessage());
    } catch (RemoteException ex) {
      cat.error("Remote exception occured.", ex);

      throw new SWFunctionalityException(ex.getMessage());
    } catch (FinderException ex) {
      cat.error("Failed to find SWBlobRecord remote interface by its primary key. Key:" + blobRecordPK, ex);

      throw new SWFunctionalityException(ex.getMessage());
    } catch (Throwable t) {
      cat.error("General exception:" + t.getMessage());

      throw new SWFunctionalityException(t.getMessage());
    } 
  } 

  /**
   * Gets functionality creation info.
   * 
   * @return Functionality creation info/
   */
  protected SWFunctionalityCreationInfo getFunctionalityCreationInfo() {
    SWFunctionalityCreationInfo res = new SWFunctionalityCreationInfo();

    res.funcName = FUNC_NAME;
    res.funcDescription = FUNC_DESCRIPTION;
    res.funcHomeClassName = FUNC_HOME_CLASS_NAME;
    res.funcHomeJndiName = FUNC_HOME_JNDI_NAME;

    SWFuncParameterCreationInfo parCreationInfo;

    parCreationInfo = new SWFuncParameterCreationInfo("boundary_size", 
            "Blob boundary size. If blob is larger than this size, it will store in the filesystem. Otherwise, it will be store in the database.", 
            "50");

    res.addParameterCreationInfo(parCreationInfo);

    parCreationInfo = new SWFuncParameterCreationInfo("fs_prefix", "Filesystem prefix", "C:/TEMP");

    res.addParameterCreationInfo(parCreationInfo);

    return res;
  } 

  private String getBlobTableName() throws RemoteException {
    String blob_TableName = table.getSWTableID().toString() + BLOB_TABLE_NAME_SUFFIX;

    return (blob_TableName);
  } 

}

