/*
 * SupportWizard 2
 * 
 * Copyright (C) 2001 ISC corp. All Rights Reserved.
 * 
 * $Id: SWBlobRecordBean.java,v 1.10 2003/07/22 23:05:31 alexi Exp $
 * Created by Michael Degtyarev (dmv@supportwizard.com)
 * Date: $Date: 2003/07/22 23:05:31 $
 * Time: $Time$
 */
package com.supportwizard.functionalities.blob.ejb;

import java.rmi.RemoteException;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.*;
import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.EJBException;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import com.supportwizard.db.ColumnList;
import com.supportwizard.db.ConditionList;
import com.supportwizard.db.SWInternal_SQL;
import com.supportwizard.db.TableList;
import com.supportwizard.dictionary.SWTable;
import com.supportwizard.dml.SWRecordPK;
import com.supportwizard.functionalities.blob.interfaces.SWBlobRecordPK;
import org.apache.log4j.Category;

/**
 * This entity bean is responsible for hte management of auxiliary attachements tables for {@link com.supportwizard.functionalities.blob.ejb.SWBlobFunctionalityBean BLOB functionality}.<p>
 * While creating BLOB functionality, auxiliary database table is being creating for user table. All further work with these tables is made thru this entity bean. We instantiate it for the concrete table passing to its <code>ejbCreate</code> an instance of <code>SWTable</code>, and thus, the main table for this bean, once set, never cannot be changed.<p>
 * Except of <code>SWTable</code> instance, there is another two persistent fields in this bean:
 * <ul>
 * <li>The name of attachment file, and
 * </li>
 * <li>The content of attachment.
 * </li>
 * </ul>
 * This bean provides setters and getters for these fields.<p>
 * See {@link com.supportwizard.functionalities.blob.ejb.SWBlobFunctionalityBean SWBlobFunctionalityBean} for further details about BLOB functionality.
 * @author Michael Degtyarev
 * @version $Revision: 1.10 $
 * 
 * @ejb:bean name="SWBlobRecord" type="BMP" jndi-name="functionalities/SWBlobRecord" transaction-type="Container"
 * @ejb:data-object container="false"
 * 
 * @ejb:resource-ref res-name="jdbc/sw" res-type="javax.sql.DataSource" res-auth="Container"
 * 
 * @ejb:finder signature="java.util.Collection findByTable(com.supportwizard.dictionary.SWTable swTable)"
 * @see SWBlobFunctionalityBean
 */
public abstract class SWBlobRecordBean implements javax.ejb.EntityBean {
  private transient Category       cat = Category.getInstance(this.getClass().getName());
  private SWTable                  table = null;
  private String                   fileName = null;
  private Object                   blobContent = null;
  private EntityContext            ctx = null;
  transient private SWBlobRecordPK swBlobRecordPK = null;
  transient private SWInternal_SQL sql = null;

  // ///////////////kos-generated
  // Michael,
  // The following 2 getters are required for correct
  // primary key class generation. Don't remove them.
  // Moreover, you can remove swBlobRecordPK field
  // and use getters and setters instead. In this
  // case uncomment those two setters, xdoclet will implement
  // them in SWBlobRecordBeanBMP.java
  // This approach is preferred now.

  /**
   * @ejb:pk-field
   * @ejb:persistent-field
   */
  public abstract Long getTableID();

  /**
   * @ejb:pk-field
   * @ejb:persistent-field
   */
  public abstract Integer getBlobID();

  // /**
  // * @ejb:persistent-field
  // */
  // public abstract void setTableID(Long tableID);
  // /**
  // * @ejb:persistent-field
  // */
  // public abstract void setBlobID(Integer blobID);
  // ///////////////

  /**
   * @ejb:create-method
   */
  public SWBlobRecordPK ejbCreate(SWTable table, String fileName, Object blobContent) throws CreateException {
    cat.debug("ENTERING ejbCreate(" + table + "," + fileName + "," + blobContent.toString() + ")");

    try {
      this.table = table;
      this.fileName = fileName;
      this.blobContent = blobContent;

      /*
       * Prepare to proceed the fo;;owing statement:
       * INSERT INTO user_table_blob VALUES (0, ?, ?);
       * 
       * Notice 1: The blob_id field of this table is both primary key and auto-increment one.
       * Notice 2: Since we are going to affect all the fields of this table, we don't use "columns" clause
       */
      HashMap params = new HashMap();
      List    values = new ArrayList();

      values.add(new Integer(0));
      values.add(fileName);
      values.add(blobContent);
      params.put("table", getBlobTableName());
      params.put("values", values);

      sql = new SWInternal_SQL();

      sql.dbConnect();

      try {
        sql.sqlDML("insert", params);

        int insertID = sql.getInsertID();

        swBlobRecordPK = new SWBlobRecordPK(table.getSWTableID(), new Integer(insertID));

        return swBlobRecordPK;
      } 
      finally {
        sql.dbDisconnect();
      } 
    } catch (Throwable t) {
      t.printStackTrace();
      cat.error(t);

      throw new CreateException(t.getMessage());
    } 
  } 

  public void ejbPostCreate(SWTable table, String fileName, Object blobContent) {}

  public void ejbRemove() throws RemoveException {
    try {

      /*
       * Prepare to proceed following statement:
       * DELETE FROM user_table_blob WHERE blob_id = ?;
       */
      HashMap       params = new HashMap();
      ConditionList where = new ConditionList();
      Integer       blobID = swBlobRecordPK.getBlobID();

      where.add("", "blob_id", "", "=", blobID, "");
      params.put("table", getBlobTableName());
      params.put("where", where);
      sql.dbConnect();

      try {
        sql.sqlDML("delete", params);
      } 
      finally {
        sql.dbDisconnect();
      } 
    } catch (Throwable t) {
      cat.error(t);

      throw new RemoveException(t.getMessage());
    } 
  } 

  public SWBlobRecordPK ejbFindByPrimaryKey(SWBlobRecordPK swBlobRecordPK) throws FinderException {
    cat.debug("ENTERING ejbFindByPrimaryKey(" + swBlobRecordPK + ")");

    try {
      Long       tableID = swBlobRecordPK.getTableID();
      String     blobTableName = getBlobTableName(swBlobRecordPK.getTableID());

      /*
       * Prepare to proceed following statement:
       * SELECT blob_id FROM user_table_blob WHERE blob_id = ?;
       */
      HashMap    params = new HashMap();
      ColumnList columns = new ColumnList();

      columns.add("blob_id");

      ConditionList where = new ConditionList();
      Integer       blobID = swBlobRecordPK.getBlobID();

      where.add("", "blob_id", "", "=", blobID, "");

      TableList from = new TableList();

      from.add(blobTableName, "");
      params.put("columns", columns);
      params.put("from", from);
      params.put("where", where);

      SWInternal_SQL sql = new SWInternal_SQL();

      sql.dbConnect();
      cat.debug("HERE !!!");

      try {
        ResultSet rs = sql.sqlQuery(params);

        if (rs.next() == false) {
          rs.close();

          throw new FinderException("Cannot find SWBlobRecord remote interface. Primary key:" + swBlobRecordPK);
        } 

        rs.close();

        return swBlobRecordPK;
      } 
      finally {
        sql.dbDisconnect();
      } 
    } catch (FinderException ex) {
      cat.warn(ex);

      throw ex;
    } catch (Throwable t) {
      cat.error(t);

      throw new FinderException(t.getMessage());
    } 
  } 

  public Collection ejbFindByTable(SWTable swTable) throws FinderException {
    cat.debug("ENTERING ejbFindByTable(" + swTable + ")");

    try {
      HashMap    params = new HashMap();
      Long       tableID = swTable.getSWTableID();

      /*
       * Prepare to proceed following statement:
       * SELECT blob_id FROM user_table_blob
       * Notice 1: Since we are going to fetch all records from specified table, we don't use "where" clause
       */
      ColumnList columns = new ColumnList();

      columns.add("blob_id");

      TableList from = new TableList();

      from.add(getBlobTableName(tableID), "");
      params.put("columns", columns);
      params.put("from", from);

      SWInternal_SQL sql = new SWInternal_SQL();

      sql.dbConnect();
      cat.debug("Connected.");

      try {
        HashSet   primaryKeys = new HashSet();
        ResultSet rs = sql.sqlQuery(params);

        while (rs.next()) {
          int            blobID_int = rs.getInt(1);
          Integer        blobID = new Integer(blobID_int);
          SWBlobRecordPK swBlobRecordPK = new SWBlobRecordPK(tableID, blobID);

          cat.debug("SWBlobRecordPK obtained:" + swBlobRecordPK);
          primaryKeys.add(swBlobRecordPK);
          cat.debug("Added.");
        } 

        rs.close();
        cat.debug("Result set closed.");

        return primaryKeys;
      } 
      finally {
        sql.dbDisconnect();
      } 
    } catch (Exception ex) {
      cat.error(ex);

      throw new FinderException(ex.getMessage());
    } 
  } 

  public void ejbLoad() {
    cat.debug("ENTERING ejbLoad");

    try {

      /*
       * Prepare to proceed following statement:
       * SELECT blob_name, blob_content FROM user_table_blob WHERE blob_id = ?;
       */
      HashMap    params = new HashMap();
      ColumnList columns = new ColumnList();

      columns.add("blob_name");
      columns.add("blob_content");

      TableList from = new TableList();

      from.add(getBlobTableName(), "");

      ConditionList where = new ConditionList();
      Integer       blobID = swBlobRecordPK.getBlobID();

      where.add("", "blob_id", "", "=", blobID, "");
      params.put("columns", columns);
      params.put("from", from);
      params.put("where", where);
      sql.dbConnect();

      try {
        ResultSet rs = sql.sqlQuery(params);

        if (rs.next() == false) {  // Requested row not found
          String errMsg = "Requested row not found. Primary key is " + swBlobRecordPK.toString();

          cat.error(errMsg);

          throw new EJBException(errMsg);
        } 

        fileName = rs.getString("blob_name");
        blobContent = rs.getObject("blob_content");

        rs.close();
      } 
      finally {
        sql.dbDisconnect();
      } 
    } catch (EJBException ex) {
      cat.error(ex);

      throw ex;
    } catch (Exception ex) {
      cat.error(ex);

      throw new EJBException(ex);
    } 
  } 

  public void ejbStore() {
    cat.debug("ENTERING ejbStore:");

    try {

      /*
       * Prepare to proceed following statement:
       * UPDATE user_table_blob SET (blob_name, blob_content) = (?, ?) WHERE blob_id = ?;
       */
      HashMap       params = new HashMap();
      String        tableName = getBlobTableName();
      ConditionList set = new ConditionList();

      set.add("", "blob_name", "", "=", fileName, "");
      set.add("", "blob_content", "", "=", blobContent, "");

      ConditionList where = new ConditionList();
      Integer       blobID = swBlobRecordPK.getBlobID();

      where.add("", "blob_id", "", "=", blobID, "");
      params.put("table", tableName);
      params.put("set", set);
      params.put("where", where);
      sql.dbConnect();

      try {
        sql.sqlDML("update", params);
      } 
      finally {
        sql.dbDisconnect();
      } 
    } catch (EJBException ex) {
      cat.error(ex);

      throw ex;
    } catch (Exception ex) {
      cat.error(ex);

      throw new EJBException(ex);
    } 
  } 

  public void ejbActivate() throws EJBException {
    try {
      swBlobRecordPK = (SWBlobRecordPK) ctx.getPrimaryKey();
      sql = new SWInternal_SQL();
    } catch (Throwable t) {
      cat.error("Error in ejbActivate.", t);

      throw new EJBException(t.getMessage());
    } 
  } 

  public void ejbPassivate() {
    sql = null;
    swBlobRecordPK = null;
  } 

  /**
   * Sets the content of BLOB for this record.
   * @ejb:interface-method view-type="remote"
   * @ejb:persistent-field
   */
  public void setBlobContent(Object blobContent) {
    this.blobContent = blobContent;
  } 

  /**
   * Gets the content of BLOB for this record.
   * @ejb:interface-method view-type="remote"
   * @ejb:persistent-field
   */
  public Object getBlobContent() {
    return blobContent;
  } 

  /**
   * Sets the name of BLOB for this record.
   * @ejb:interface-method view-type="remote"
   * @ejb:persistent-field
   */
  public void setFileName(String fileName) {
    this.fileName = fileName;
  } 

  /**
   * Gets the name of BLOB for this record.
   * @ejb:interface-method view-type="remote"
   * @ejb:persistent-field
   */
  public String getFileName() {
    return fileName;
  } 

  public void setEntityContext(javax.ejb.EntityContext ctx) {
    this.ctx = ctx;
  } 

  public void unsetEntityContext() {
    ctx = null;
  } 

  private String getBlobTableName(Long tableID) throws FinderException, RemoteException, NamingException {
    return tableID.toString() + SWBlobFunctionalityBean.BLOB_TABLE_NAME_SUFFIX;
  } 

  private String getBlobTableName() throws FinderException, RemoteException, NamingException {
    String blobTableName = getBlobTableName(table.getSWTableID());

    return blobTableName;
  } 

}

